@@ -230,9 +230,9 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
             break;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=p->red;
-            q->green=p->green;
-            q->blue=p->blue;
+            SetRedPixelComponent(q,GetRedPixelComponent(p));
+            SetGreenPixelComponent(q,GetGreenPixelComponent(p));
+            SetBluePixelComponent(q,GetBluePixelComponent(p));
             p++;
             q++;
           }
