@@ -17,7 +17,7 @@
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -99,35 +99,31 @@ MagickExport Image *BorderImage(const Image *image,
   const RectangleInfo *border_info,ExceptionInfo *exception)
 {
   Image
-    *border_image;
+    *border_image,
+    *clone_image;
+
+  FrameInfo
+    frame_info;
 
-  /*
-    Decorate the image with a border.
-  */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(border_info != (const RectangleInfo *) NULL);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  border_image=CloneImage(image,image->columns+2*border_info->width,
-    image->rows+2*border_info->height,MagickTrue,exception);
-  if (border_image == (Image *) NULL)
+  assert(border_info != (RectangleInfo *) NULL);
+  frame_info.width=image->columns+(border_info->width << 1);
+  frame_info.height=image->rows+(border_info->height << 1);
+  frame_info.x=(long) border_info->width;
+  frame_info.y=(long) border_info->height;
+  frame_info.inner_bevel=0;
+  frame_info.outer_bevel=0;
+  clone_image=CloneImage(image,0,0,MagickTrue,exception);
+  if (clone_image == (Image *) NULL)
     return((Image *) NULL);
-  if (SetImageStorageClass(border_image,DirectClass) == MagickFalse)
-    {
-      InheritException(exception,&border_image->exception);
-      border_image=DestroyImage(border_image);
-      return((Image *) NULL);
-    }
-  border_image->background_color=border_image->border_color;
-  if (border_image->background_color.opacity != OpaqueOpacity)
-    border_image->matte=MagickTrue;
-  (void) SetImageBackgroundColor(border_image);
-  (void) CompositeImage(border_image,image->compose,image,(long)
-    border_info->width,(long) border_info->height);
-  border_image->background_color=image->background_color;
+  clone_image->matte_color=image->border_color;
+  border_image=FrameImage(clone_image,&frame_info,exception);
+  clone_image=DestroyImage(clone_image);
+  if (border_image != (Image *) NULL)
+    border_image->matte_color=image->matte_color;
   return(border_image);
 }
 
@@ -167,6 +163,10 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
 {
 #define FrameImageTag  "Frame/Image"
 
+  CacheView
+    *image_view,
+    *frame_view;
+
   Image
     *frame_image;
 
@@ -194,9 +194,6 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     height,
     width;
 
-  CacheView
-    *frame_view;
-
   /*
     Check frame geometry.
   */
@@ -285,19 +282,20 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     }
   status=MagickTrue;
   progress=0;
+  image_view=AcquireCacheView(image);
   frame_view=AcquireCacheView(frame_image);
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
   if (height != 0)
     {
       register IndexPacket
-        *__restrict frame_indexes;
+        *restrict frame_indexes;
 
       register long
         x;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
 
       /*
         Draw top of ornamental border.
@@ -402,18 +400,18 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     Draw sides of ornamental border.
   */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status)
+  #pragma omp parallel for schedule(static) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *__restrict frame_indexes;
+      *restrict frame_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     /*
       Initialize scanline with matte color.
@@ -449,12 +447,39 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     /*
       Set frame interior to interior color.
     */
-    for (x=0; x < (long) image->columns; x++)
-    {
-      SetPixelPacket(frame_image,&interior,q,frame_indexes);
-      q++;
-      frame_indexes++;
-    }
+    if ((image->compose != CopyCompositeOp) &&
+        ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
+      for (x=0; x < (long) image->columns; x++)
+      {
+        SetPixelPacket(frame_image,&interior,q,frame_indexes);
+        q++;
+        frame_indexes++;
+      }
+    else
+      {
+        register const IndexPacket
+          *indexes;
+
+        register const PixelPacket
+          *p;
+
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+        if (p == (const PixelPacket *) NULL)
+          {
+            status=MagickFalse;
+            continue;
+          }
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
+        (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
+        if ((image->colorspace == CMYKColorspace) &&
+            (frame_image->colorspace == CMYKColorspace))
+          {
+            (void) CopyMagickMemory(frame_indexes,indexes,image->columns*
+              sizeof(*indexes));
+            frame_indexes+=image->columns;
+          }
+        q+=image->columns;
+      }
     for (x=0; x < (long) frame_info->inner_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -494,13 +519,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   if (height != 0)
     {
       register IndexPacket
-        *__restrict frame_indexes;
+        *restrict frame_indexes;
 
       register long
         x;
 
       register PixelPacket
-        *__restrict q;
+        *restrict q;
 
       /*
         Draw bottom of ornamental border.
@@ -601,11 +626,16 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
         }
     }
   frame_view=DestroyCacheView(frame_view);
-  x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
-    frame_info->inner_bevel);
-  y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
-    frame_info->inner_bevel);
-  (void) CompositeImage(frame_image,image->compose,image,x,y);
+  image_view=DestroyCacheView(image_view);
+  if ((image->compose != CopyCompositeOp) &&
+      ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
+    {
+      x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
+        frame_info->inner_bevel);
+      y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
+        frame_info->inner_bevel);
+      (void) CompositeImage(frame_image,image->compose,image,x,y);
+    }
   return(frame_image);
 }
 
@@ -649,6 +679,9 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
 #define RaiseImageTag  "Raise/Image"
 #define TroughFactor  ScaleCharToQuantum(135)
 
+  CacheView
+    *image_view;
+
   ExceptionInfo
     *exception;
 
@@ -663,9 +696,6 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     foreground,
     background;
 
-  CacheView
-    *image_view;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -692,7 +722,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
   exception=(&image->exception);
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status)
+  #pragma omp parallel for schedule(static) shared(progress,status)
 #endif
   for (y=0; y < (long) raise_info->height; y++)
   {
@@ -700,7 +730,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -712,37 +742,37 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
     for (x=0; x < y; x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
       q++;
     }
     for ( ; x < (long) (image->columns-y); x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*
         AccentuateFactor+(MagickRealType) foreground*(QuantumRange-
         AccentuateFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         AccentuateFactor+(MagickRealType) foreground*(QuantumRange-
         AccentuateFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         AccentuateFactor+(MagickRealType) foreground*(QuantumRange-
         AccentuateFactor)));
       q++;
     }
     for ( ; x < (long) image->columns; x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*ShadowFactor+
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*ShadowFactor+
         (MagickRealType) background*(QuantumRange-ShadowFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
@@ -759,7 +789,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=(long) raise_info->height; y < (long) (image->rows-raise_info->height); y++)
   {
@@ -767,7 +797,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -779,13 +809,13 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
     for (x=0; x < (long) raise_info->width; x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
       q++;
@@ -794,11 +824,11 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       q++;
     for ( ; x < (long) image->columns; x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*ShadowFactor+
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*ShadowFactor+
         (MagickRealType) background*(QuantumRange-ShadowFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
@@ -815,7 +845,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=(long) (image->rows-raise_info->height); y < (long) image->rows; y++)
   {
@@ -823,7 +853,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *__restrict q;
+      *restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -835,34 +865,34 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
     for (x=0; x < (long) (image->rows-y); x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         HighlightFactor+(MagickRealType) foreground*(QuantumRange-
         HighlightFactor)));
       q++;
     }
     for ( ; x < (long) (image->columns-(image->rows-y)); x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*TroughFactor+
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*TroughFactor+
         (MagickRealType) background*(QuantumRange-TroughFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         TroughFactor+(MagickRealType) background*(QuantumRange-TroughFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         TroughFactor+(MagickRealType) background*(QuantumRange-TroughFactor)));
       q++;
     }
     for ( ; x < (long) image->columns; x++)
     {
-      q->red=RoundToQuantum(QuantumScale*((MagickRealType) q->red*ShadowFactor+
+      q->red=ClampToQuantum(QuantumScale*((MagickRealType) q->red*ShadowFactor+
         (MagickRealType) background*(QuantumRange-ShadowFactor)));
-      q->green=RoundToQuantum(QuantumScale*((MagickRealType) q->green*
+      q->green=ClampToQuantum(QuantumScale*((MagickRealType) q->green*
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
-      q->blue=RoundToQuantum(QuantumScale*((MagickRealType) q->blue*
+      q->blue=ClampToQuantum(QuantumScale*((MagickRealType) q->blue*
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
